@@ -7,6 +7,7 @@
 #include <sys/time.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <limits.h>
 #include <unistd.h>
 
 #include "trace-cmd-private.h"
@@ -77,6 +78,16 @@ static ssize_t  write_fd(int fd, const void *data, size_t size)
 	return tot;
 }
 
+static int read_size_val(struct tep_handle *tep, size_t size, int *endian4)
+{
+	if (size > UINT_MAX)
+		return -1;
+
+	*endian4 = size;
+	*endian4 = tep_read_number(tep, endian4, 4);
+	return 0;
+}
+
 static ssize_t  do_write(struct tracecmd_compression *handle,
 			  const void *data, size_t size)
 {
@@ -121,9 +132,12 @@ off_t tracecmd_compress_lseek(struct tracecmd_compression *handle, off_t offset,
 {
 	unsigned long p;
 
-	if (!handle || !handle->buffer)
+	if (!handle)
 		return (off_t)-1;
 
+	if (!handle->buffer)
+		return (whence == SEEK_CUR && offset == 0) ? 0 : (off_t)-1;
+
 	switch (whence) {
 	case SEEK_CUR:
 		p = handle->pointer + offset;
@@ -331,7 +345,8 @@ int tracecmd_compress_block(struct tracecmd_compression *handle)
 		goto out;
 
 	/* Write uncompressed data size */
-	endian4 = tep_read_number(handle->tep, &handle->pointer, 4);
+	if (read_size_val(handle->tep, handle->pointer, &endian4) < 0)
+		goto out;
 	ret = do_write(handle, &endian4, 4);
 	if (ret != 4) {
 		ret = -1;
@@ -351,6 +366,32 @@ out:
 	free(buf);
 	return ret;
 }
+/**
+ * tracecmd_uncompress_buffer - Returned the current uncompressed buffer
+ * @handle: compression handle
+ * @size: A pointer to place the size of the uncompressed buffer in
+ *
+ * Returns the current uncompressed buffer and resets the compression
+ * @handle.
+ *
+ * Returns the buffer (must be freed with free()) or NULL.
+ *  @size (if not NULL) will contain the size of the uncompressed buffer.
+ */
+void *tracecmd_uncompress_buffer(struct tracecmd_compression *handle, size_t *size)
+{
+	void *buffer;
+
+	if (!handle)
+		return NULL;
+
+	if (size)
+		*size = handle->capacity;
+
+	buffer = handle->buffer;
+	handle->buffer = NULL;
+	tracecmd_compress_reset(handle);
+	return buffer;
+}
 
 /**
  * tracecmd_compress_buffer_write - write() to compression buffer
@@ -703,7 +744,8 @@ int tracecmd_compress_copy_from(struct tracecmd_compression *handle, int fd, int
 
 	/* save the initial offset and write 0 as initial chunk count */
 	offset = lseek(handle->fd, 0, SEEK_CUR);
-	write_fd(handle->fd, &chunks, 4);
+	if (write_fd(handle->fd, &chunks, 4) != 4)
+		return -1;
 
 	do {
 		all = 0;
@@ -735,13 +777,15 @@ int tracecmd_compress_copy_from(struct tracecmd_compression *handle, int fd, int
 			}
 			size = ret;
 			/* Write compressed data size */
-			endian4 = tep_read_number(handle->tep, &size, 4);
+			if (read_size_val(handle->tep, size, &endian4) < 0)
+				break;
 			ret = write_fd(handle->fd, &endian4, 4);
 			if (ret != 4)
 				break;
 
 			/* Write uncompressed data size */
-			endian4 = tep_read_number(handle->tep, &all, 4);
+			if (read_size_val(handle->tep, all, &endian4) < 0)
+				break;
 			ret = write_fd(handle->fd, &endian4, 4);
 			if (ret != 4)
 				break;
@@ -763,9 +807,11 @@ int tracecmd_compress_copy_from(struct tracecmd_compression *handle, int fd, int
 	if (lseek(handle->fd, offset, SEEK_SET) == (off_t)-1)
 		return -1;
 
-	endian4 = tep_read_number(handle->tep, &chunks, 4);
+	if (read_size_val(handle->tep, chunks, &endian4) < 0)
+		return -1;
 	/* write chunks count*/
-	write_fd(handle->fd, &chunks, 4);
+	if (write_fd(handle->fd, &endian4, 4) != 4)
+		return -1;
 	end_offset = lseek(handle->fd, 0, SEEK_END);
 	if (end_offset == (off_t)-1)
 		return -1;
@@ -974,7 +1020,8 @@ int tracecmd_uncompress_copy_to(struct tracecmd_compression *handle, int fd,
 		if (ret < 0)
 			break;
 
-		write_fd(fd, bytes_out, ret);
+		if (write_fd(fd, bytes_out, ret) != ret)
+			break;
 		wsize += ret;
 		chunks--;
 	}
