@@ -6,10 +6,12 @@
 
 #include <stdlib.h>
 #include <sys/stat.h>
+#include <unistd.h>
 
 #include "tracefs.h"
 #include "trace-local.h"
 
+#define BTF_FILE	"/sys/kernel/btf/vmlinux"
 
 static void dump_file_content(const char *path)
 {
@@ -389,14 +391,22 @@ static void show_tracers(void)
 	show_file("available_tracers");
 }
 
-void show_options(const char *prefix, struct buffer_instance *buffer)
+void show_options(const char *prefix, struct buffer_instance *buffer, const char *re)
 {
 	struct tracefs_instance *instance = buffer ? buffer->tracefs : NULL;
 	struct dirent *dent;
 	struct stat st;
+	regex_t reg;
 	char *path;
 	DIR *dir;
 
+	if (re && strlen(re)) {
+		if (regcomp(&reg, re, REG_ICASE|REG_NOSUB))
+			die("invalid function regex '%s'", re);
+	} else {
+		re = NULL;
+	}
+
 	if (!prefix)
 		prefix = "";
 
@@ -423,6 +433,9 @@ void show_options(const char *prefix, struct buffer_instance *buffer)
 		    strcmp(name, "..") == 0)
 			continue;
 
+		if (re && regexec(&reg, name, 0, NULL, 0) != 0)
+			continue;
+
 		ret = asprintf(&file, "options/%s", name);
 		if (ret < 0)
 			die("Failed to allocate file name");
@@ -463,17 +476,71 @@ static void show_clocks(void)
 	free(clocks);
 }
 
-
-static void show_functions(const char *funcre)
+static struct tep_handle *load_btf(void)
 {
+	struct tep_handle *tep;
+	struct stat st;
+	ssize_t size;
+	char *buf;
+	int ret;
+	int fd;
+	int r, s = 0;
+
+	ret = stat(BTF_FILE, &st);
+	if (ret < 0)
+		return NULL;
+
+	size = st.st_size;
+	buf = malloc(size);
+	if (!buf)
+		return NULL;
+
+	fd = open(BTF_FILE, O_RDONLY);
+	if (fd < 0)
+		return NULL;
+
+	while (size) {
+		r = read(fd, buf + s, size);
+		if (r < 0)
+			break;
+		s += r;
+		size -= r;
+	}
+	close(fd);
+
+	tep = tep_alloc();
+
+	if (!tep || tep_load_btf(tep, buf, s) < 0) {
+		tep_free(tep);
+		tep = NULL;
+	}
+
+	free(buf);
+	return tep;
+}
+
+static void show_functions(const char *funcre, int params)
+{
+	struct tep_handle *tep = NULL;
+	struct trace_seq s;
 	bool found = false;
 	char *new_re = NULL;
 	char **list;
 	int i;
 
 	if (!funcre) {
-		show_file("available_filter_functions");
-		return;
+		if (!params) {
+			show_file("available_filter_functions");
+			return;
+		}
+		funcre = "^.*";
+	}
+
+	trace_seq_init(&s);
+	if (params) {
+		tep = load_btf();
+		if (!tep)
+			params = 0;
 	}
 
 	/* if the re doesn't have any regular expressions, then add them */
@@ -499,8 +566,18 @@ static void show_functions(const char *funcre)
 
 	if (tracefs_filter_functions(funcre, NULL, &list) < 0)
 		die("Failed to read filte functions");
-	for (i = 0; list && list[i]; i++)
-		printf("%s\n", list[i]);
+	for (i = 0; list && list[i]; i++) {
+		printf("%s", list[i]);
+		if (params) {
+			trace_seq_reset(&s);
+			if (tep_btf_list_args(tep, &s, list[i]) >= 0) {
+				printf("(");
+				trace_seq_do_printf(&s);
+				printf(")");
+			}
+		}
+		printf("\n");
+	}
 	tracefs_list_free(list);
 	free(new_re);
 }
@@ -641,10 +718,12 @@ void trace_list(int argc, char **argv)
 	int systems = 0;
 	int show_all = 1;
 	int compression = 0;
+	int params = 0;
 	int i;
 	const char *arg;
 	const char *funcre = NULL;
 	const char *eventre = NULL;
+	const char *optionre = NULL;
 
 	for (i = 2; i < argc; i++) {
 		arg = NULL;
@@ -694,6 +773,7 @@ void trace_list(int argc, char **argv)
 				break;
 			case 'o':
 				options = 1;
+				optionre = arg;
 				show_all = 0;
 				break;
 			case 'f':
@@ -718,6 +798,10 @@ void trace_list(int argc, char **argv)
 					flags |= SHOW_EVENT_FULL;
 					break;
 				}
+				if (strcmp(argv[i], "--proto") == 0) {
+					params = 1;
+					break;
+				}
 				fprintf(stderr, "list: invalid option -- '%s'\n",
 					argv[i]);
 			default:
@@ -735,7 +819,7 @@ void trace_list(int argc, char **argv)
 		show_tracers();
 
 	if (options)
-		show_options(NULL, NULL);
+		show_options(NULL, NULL, optionre);
 
 	if (plug)
 		show_plugins();
@@ -744,7 +828,7 @@ void trace_list(int argc, char **argv)
 		show_plugin_options();
 
 	if (funcs)
-		show_functions(funcre);
+		show_functions(funcre, params);
 
 	if (buffers)
 		show_buffers();
@@ -763,7 +847,7 @@ void trace_list(int argc, char **argv)
 		printf("\ntracers:\n");
 		show_tracers();
 		printf("\noptions:\n");
-		show_options(NULL, NULL);
+		show_options(NULL, NULL, NULL);
 		show_compression();
 	}
 
